/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.allgemein.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.Properties;

import de.bsvrz.kex.datex2.allgemein.KonfigurationsModul;

/**
 * Klasse zur Implementierung eines Konfigurationsmoduls durch Auslesen von
 * Dateien im JavaProperty-Format.
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class EinfachesKonfigurationsModul implements KonfigurationsModul {
	
  /** Lokale Speicherung der Konfigurationsdaten. */
  private Properties properties;
  
  /** Kontextname. */
	private String context;
	 
	/**
	 * Konstruktor. Der Konfigurationskontext wird durch Angabe einer Datei initialisiert, die
	 * mit Aufruf des Konstruktors auch eingelesen und verarbeitet wird.
	 * 
	 * @param propertyFile Referenz auf Datei
	 * @throws IOException Ein-/Ausgabefehler
	 */
	 public EinfachesKonfigurationsModul(final File propertyFile) throws IOException {
     properties = new Properties();
     InputStream inputStream = new FileInputStream(propertyFile);
     properties.load(inputStream);
     
     // entferne Dateiendung
     
     String myContext = propertyFile.getName();
     int dot = myContext.lastIndexOf('.');
     if (dot > 0) {
       myContext = myContext.substring(0, dot);
     }
     this.context = myContext;
	 }

	  @Override
	  public boolean hasData(final String key) {
	    return properties.getProperty(key) != null;
	  }
	 
	 @Override
	 public String getContext() {
	   return context;
	 }
	
	@Override
	public <T extends Serializable> T getData(final String key, final Class<T> classx) {
	  if (classx.equals(String.class)) {
	    return classx.cast(properties.getProperty(key));	    
	  }
    if (classx.equals(Integer.class)) {
	    try {
	      int i = Integer.parseInt(properties.getProperty(key));
	      return classx.cast(new Integer(i));
	    } catch (NumberFormatException ex) {
	      throw new IllegalArgumentException("Fr den Schlssel '" + key + "' wurde eine Zahl erwartet.");
	    }
	  }
		return classx.cast(properties.getProperty(key));
	}

	@Override
	public String getString(final String key) {
		return properties.getProperty(key);
	}

	@Override
	public Integer getInteger(final String key) {
	  if (!hasData(key)) {
	    return null;
	  } else {
	  	return new Integer(properties.getProperty(key));
	  }
	}
}
