/**
 * SWE KEx-DatexII (de.bsvrz.kex.datex2)
 * 
 * Copyright (C) 2014 TraffGo Road GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * TraffGo Road GmbH
 * Ostwall 11-19
 * 47798 Krefeld
 * Phone: +49-2151-3295058
 * mailto: info@traffgoroad.de
 */

package de.bsvrz.kex.datex2.plugins.definition.impl;
import javax.naming.ConfigurationException;

import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.ausfuehrung.impl.StandardExportPluginAusfuehrung;
import de.bsvrz.kex.datex2.plugins.datensammlung.impl.ImmerLetztesElement;
import de.bsvrz.kex.datex2.plugins.definition.PluginExport;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.TMCMeldungen;
import de.bsvrz.kex.datex2.plugins.publizierung.impl.MDMPublizierung;
import de.bsvrz.kex.datex2.plugins.publizierung.impl.XMLFilePublizierung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Plugin-Definitions-Klasse ("assembly class") fr das Plugin DaV-MDM-009 (Verkehrsmeldungen).
 * 
 * @author TraffGo Road GmbH
 * @version $Id$
 */

public class DaV_MDM_009 extends PluginExport {
  
  /** Name der Menge fr Meldungen. */
  private static final String RDS_MELDUNGEN_MENGE = "RDSMeldungen";

  /** Zu behandelnde Attributgruppe. */
  private static final String ATTRIBUT_GRUPPE = "atg.rdsMeldung";
  
  /** Logging. */
  private static final Debug LOGGER = Debug.getLogger();

  /** Standard-Periode des Plugins (in Sekunden). */
  private static final int STANDARD_PERIOD = 60;
  
  /**
   * Konstruktor.
   */
  public DaV_MDM_009() {
    super(STANDARD_PERIOD);
  }


  @Override
  public void initialisiere() throws ConfigurationException {
    LOGGER.finer("Plugin-Klasse " + this.getClass().getName() + " wird initialisiert.");
    ConfigurationObject lmsObjekt = null;
    String lmsObjektString = konfigurationsModul.getString("LMS-Objekt");
    String logPrefix = "In Kontext '" + konfigurationsModul.getContext() + "': ";
    if (lmsObjektString == null) {
      throw new ConfigurationException(logPrefix + "Konfigurationsdatum 'LMS-Objekt' wurde nicht angegeben.");
    }
    try {
      lmsObjekt = (ConfigurationObject) davConnection.getDataModel().getObject(lmsObjektString);
      if (lmsObjekt == null) {
        throw new ConfigurationException(logPrefix + "Angegebenes LMS-Objekt konnte nicht gefunden werden.");           
      }
    } catch (ClassCastException ex) {
      throw new ConfigurationException(logPrefix + "Angegebenes LMS-Objekt ist kein Konfigurationsobjekt."); 
    }

    String aspListString = konfigurationsModul.getString("Aspekte");
    if (aspListString == null) {
      throw new ConfigurationException(logPrefix + "Konfigurationsdatum 'Aspekte' wurde nicht angegeben.");
    }
    
    for (String asp : aspListString.split(";")) {
      ObjekteAtgAsp rds = new ObjekteAtgAsp(davConnection, lmsObjekt, RDS_MELDUNGEN_MENGE, ATTRIBUT_GRUPPE, asp);
      objekte.add(rds); 
    }
    
    initialisiereAbhaengigkeiten();
  }
  
  @Override
  public void instantiiereDynamischePublizierungsMethode() {
    dynamischePublizierungsMethoden.add(new MDMPublizierung(this, "dynamisch"));
    dynamischePublizierungsMethoden.add(new XMLFilePublizierung(this, "dynamisch"));
  }

  @Override
  public void instantiiereStatischePublizierungsMethode() {
  }

  @Override
  public void instantiiereDatexKonvertierungStatisch() {
  	this.datexKonvertierungStatisch = null;    
  }

  @Override
  public void instantiiereDatexKonvertierungDynamisch() {
   this.datexKonvertierungDynamisch = new TMCMeldungen(this);       
  }

  @Override
  public void instantiierePluginAusfuehrungsAlgorithmus() {
    this.pluginAusfuehrungsAlgorithmus = new StandardExportPluginAusfuehrung(this);    
  }

  @Override
  public void instantiiereDatenSammlungsMethode() {
    this.datenSammlungsMethode = new ImmerLetztesElement(this);
  }
}
