package de.bsvrz.datex2.test.cases;
import java.util.Arrays;
import java.util.List;

import javax.naming.ConfigurationException;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.datex2.test.factories.CreateMockUmfelddaten;
import de.bsvrz.datex2.test.mockclasses.MockBrokenKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockDavConnection;
import de.bsvrz.datex2.test.mockclasses.MockKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockPids;
import de.bsvrz.datex2.test.mockclasses.MockPublishMethod;
import de.bsvrz.kex.datex2.plugins.definition.impl.DaV_MDM_007;

public class WhiteBox_PlugIn_DaV_MDM_007 extends WhiteBox_PlugIns {
  public WhiteBox_PlugIn_DaV_MDM_007() {
    super("DATEXII_DaV-MDM-007_dyn.xsd", "DATEXII_DaV-MDM-007_stat.xsd");
  }
    
  List<Float> listFloatData = Arrays.asList(0.2f, 0.6f, 1.0f, 0.0f, -1.0f, -123.0f, Float.NaN, Float.NEGATIVE_INFINITY, Float.POSITIVE_INFINITY, 0.3f);

  List<String> listeNiederschlagsArt = Arrays.asList("nicht ermittelbar", "fehlerhaft", "nicht ermittelbar/fehlerhaft",
      "kein Niederschlag", "Niederschlag aller Art", "Leichter/mittlerer Niederschlag", "starker Niederschlag",
      "Sprhregen", "Sprhregen WMO1", "Sprhregen WMO2", "Sprhregen WMO3", "Sprhregen WMO4", "Sprhregen WMO5",
      "Sprhregen WMO6", "Sprhregen WMO7", "Sprhregen WMO8", "Sprhregen WMO9", "Regen", "Regen WMO1", "Regen WMO2",
      "Regen WMO3", "Regen WMO4", "Regen WMO5", "Regen WMO6", "Regen WMO7", "Regen WMO8", "Regen WMO9",
      "SchneeG/gefrorener Niederschlag", "Schnee WMO1", "Schnee WMO2", "Schnee WMO3", "Graupel WMO1", "Graupel WMO2",
      "Graupel WMO3", "Hagel WMO1", "Hagel WMO2", "Hagel WMO3");

  List<String> listeFahrbahnZustand = Arrays.asList("nicht ermittelbar", "fehlerhaft","nicht ermittelbar/fehlerhaft",
      "trocken", "feucht", "nass", "gefrorenes Wasser", "Schnee/Schneematsch", "Eis", "Raureif");
  
  /**
   * Standard Gutlauf
   * 
   * @throws Exception
   */
  @Test
  public void testGutlauf() throws Exception {
    CreateMockUmfelddaten ufdCreation = new CreateMockUmfelddaten();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport, davConnection, ufdCreation, staticResult, dynamicResult);

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CANNOT_BE_NULL));

    for (String niederschlagsArt : listeNiederschlagsArt) {
      ufdCreation.mockSendData(davConnection, MockPids.TYP_NIEDERSCHLAGSART, ufdCreation.createTextData(MockPids.DS_NIEDERSCHLAGSART, niederschlagsArt));      
      mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
      
      Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));
    }
    
    for (String fahrbahnZustand : listeFahrbahnZustand) {
      ufdCreation.mockSendData(davConnection, MockPids.TYP_FAHRBAHNOBERFLAECHENZUSTAND, ufdCreation.createTextData(MockPids.DS_FAHRBAHNOBERFLAECHENZUSTAND, fahrbahnZustand));      
      mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
      
      Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));
    }

    for (int degree = 0; degree < 360; degree += 3) {
      ufdCreation.mockSendNumericData(davConnection, MockPids.TYP_WINDRICHTUNG, MockPids.DS_WINDRICHTUNG, degree);            
      mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();      
      Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));
    }

    ufdCreation.mockSendDataToAll(davConnection, 0.3f, "kein Niederschlag", "trocken");

    mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
    
    Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));
  }

  /**
   * Testlauf mit typischen fehlerhaften Werten (siehe Float-Liste oben).
   * 
   * @throws Exception
   */

  @Test
  public void testFehlerhafteDaten() throws Exception {
    CreateMockUmfelddaten ufdCreation = new CreateMockUmfelddaten();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport, davConnection, ufdCreation, staticResult, dynamicResult);

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CANNOT_BE_NULL));

    for (Float floatData : listFloatData) {
      ufdCreation.mockSendDataToAll(davConnection, floatData, floatData.toString(), floatData.toString());
      mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
      
      Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CAN_BE_NULL));
    }   
  }

  /**
   * Fehlende obligatorische Parameter mssen zu einer Exception fhren.
   * 
   * @throws Exception
   */
  
  @Test(expected=ConfigurationException.class)
  public void testFehlendeParameterStatisch() throws Exception {
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport);

    MockKonfigUndPersistenzModul konfig = (MockKonfigUndPersistenzModul) mockPluginExport.getKonfigurationsModul();
    konfig.removeData("National-Identifier");
    mockPluginExport.initialisiere();
  }

  /**
   * Fehlende obligatorische Parameter mssen zu einer Exception fhren.
   * 
   * @throws Exception
   */
 
  @Test(expected=ConfigurationException.class)
  public void testFehlendeParameterStatisch2() throws Exception {
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport);

    MockKonfigUndPersistenzModul konfig = (MockKonfigUndPersistenzModul) mockPluginExport.getKonfigurationsModul();
    konfig.removeData("LocationGroupId");
    mockPluginExport.initialisiere();
  }
 
  /**
   * Fehlende obligatorische Parameter mssen zu einer Exception fhren.
   * 
   * @throws Exception
   */
 
  @Test(expected=ConfigurationException.class)
  public void testFehlendeParameterStatisch3() throws Exception {
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport);

    MockKonfigUndPersistenzModul konfig = (MockKonfigUndPersistenzModul) mockPluginExport.getKonfigurationsModul();
    konfig.removeData("Maximales-Alter");
    mockPluginExport.initialisiere();
  }
 
  /**
   * Fehlende optionale Parameter drfen nicht zu einer Nicht-Publikation fhren.
   * 
   * @throws Exception
   */
  
  @Test
  public void testFehlendeOptionaleParameter() throws Exception {
    CreateMockUmfelddaten ufdCreation = new CreateMockUmfelddaten();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport, davConnection, ufdCreation, staticResult, dynamicResult);

    MockKonfigUndPersistenzModul konfig = (MockKonfigUndPersistenzModul) mockPluginExport.getKonfigurationsModul();
    konfig.removeData("StandardIntervall");

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CANNOT_BE_NULL));

    ufdCreation.mockSendDataToAll(davConnection, 0.3f, "kein Niederschlag", "trocken");

    mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
    
    Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));

  }  

  /**
   * Trotz fehlender Persistenz-Schicht sollten noch immer Daten abgeliefert werden. 
   * 
   * @throws Exception
   */
  @Test
  public void testBrokenPersistence() throws Exception {
    CreateMockUmfelddaten ufdCreation = new CreateMockUmfelddaten();
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport, davConnection, ufdCreation, staticResult, dynamicResult);

    mockPluginExport.setPersistenzModul(new MockBrokenKonfigUndPersistenzModul());

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CAN_BE_NULL));

    ufdCreation.mockSendDataToAll(davConnection, 0.3f, "kein Niederschlag", "trocken");

    mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
    
    Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CANNOT_BE_NULL));
  }

  /**
   * Wenn keine Umfelddatensensoren und -messstellen definiert sind, kann zwar kein XML erzeugt werden (= leeres Dokument),
   * die Exceptions mssen aber in den aufgerufenen Methoden abgefangen werden.
   * 
   * @throws Exception
   */
  
  @Test
  public void testKeineUmfelddatensensoren() throws Exception {
    MockDavConnection davConnection = new MockDavConnection(null, null);
    MockPublishMethod dynamicResult = new MockPublishMethod();
    MockPublishMethod staticResult = new MockPublishMethod();
    
    DaV_MDM_007 mockPluginExport = new DaV_MDM_007();

    init(mockPluginExport, davConnection, null, staticResult, dynamicResult);

    mockPluginExport.initialisiere();
    
    mockPluginExport.getPluginAusfuehrungsAlgorithmus().starte();

    Assert.assertTrue(validate(SCHEMA_URI_STAT, mockPluginExport.getDatexKonvertierungStatisch().getJAXBContext(), staticResult.result , CanBeNull.CAN_BE_NULL));

    mockPluginExport.getPluginAusfuehrungsAlgorithmus().aktualisiere();
    
    Assert.assertTrue(validate(SCHEMA_URI_DYN, mockPluginExport.getDatexKonvertierungDynamisch().getJAXBContext(), dynamicResult.result , CanBeNull.CAN_BE_NULL));    
  }

  @Override
  protected void populateConfigAndPersistence(MockKonfigUndPersistenzModul konfig, MockKonfigUndPersistenzModul persistenz) {
    konfig.putData("National-Identifier", "TestNI");
    konfig.putData("LocationGroupId", "TestGroupId");
    konfig.putData("StandardIntervall", "3.0");
    konfig.putData("Maximales-Alter", 3);
    persistenz.putData("HashCodeStaticXML", 123);
    persistenz.putData("PredefinedLocationVersion", "correct");
  }

}
