package de.bsvrz.datex2.test.cases.old;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;


public class InjiziereTestdaten001 implements ClientSenderInterface, ClientReceiverInterface {
  private static final String ASP_MQ_VERKEHRSDATEN = "asp.agregation1Minute";
  private static final String ATG_MQ_VERKEHRSDATEN = "atg.verkehrsDatenKurzZeitMq";
  private static final String ATG_MQ_ZUSTAND = "atg.strfallZustand";
  private static final String ASP_MQ_ZUSTAND = "asp.strfallVerfahrenMARZ";
  private static final String ASP_FS_VERKEHRSDATEN = "asp.externeErfassung";
  private static final String ATG_FS_VERKEHRSDATEN = "atg.verkehrsDatenKurzZeitIntervall";
  private static final String ATG_FS_ZUSTAND = "atg.strfallZustand";
  private static final String ASP_FS_ZUSTAND = "asp.strfallVerfahrenMARZ";
  private static final String ASP_UFDS = "asp.externeErfassung";
  ClientDavConnection davConnection;
  
  public void initialize(String[] args) throws MissingParameterException, CommunicationError, ConnectionException, InconsistentLoginException {
    ClientDavParameters parameters = new ClientDavParameters(args);
    davConnection = new ClientDavConnection(parameters);
    davConnection.connect();
    davConnection.login();
  }

  public void sendData(String pid, String atg, String asp, Data d) throws DataNotSubscribedException, SendSubscriptionNotConfirmed {
    DataDescription dd = new DataDescription(davConnection.getDataModel().getAttributeGroup(atg), davConnection.getDataModel().getAspect(asp));
    ResultData data = new ResultData(davConnection.getDataModel().getObject(pid), dd, davConnection.getTime(), d);
    davConnection.sendData(data);
  }

  public void sendData(SystemObject obj, AttributeGroup atg, Aspect asp, Data d) throws DataNotSubscribedException, SendSubscriptionNotConfirmed {
    DataDescription dd = new DataDescription(atg, asp);
    ResultData data = new ResultData(obj, dd, davConnection.getTime(), d);
    davConnection.sendData(data);
  }

  public void setStatusAndQS(Data d) {
    d.getScaledValue("Wert").set(17);
    d.getItem("Gte").getItem("Index").asScaledValue().set(0.9);
    d.getItem("Gte").getItem("Verfahren").asTextValue().setText("Standard");

    d.getItem("Status").getItem("Erfassung").getTextValue("NichtErfasst").setText("Nein");
    d.getItem("Status").getItem("PlFormal").getTextValue("WertMax").setText("Nein");
    d.getItem("Status").getItem("PlFormal").getTextValue("WertMin").setText("Nein");
    d.getItem("Status").getItem("PlLogisch").getTextValue("WertMaxLogisch").setText("Nein");
    d.getItem("Status").getItem("PlLogisch").getTextValue("WertMinLogisch").setText("Nein");
    d.getItem("Status").getItem("MessWertErsetzung").getTextValue("Implausibel").setText("Nein");
    d.getItem("Status").getItem("MessWertErsetzung").getTextValue("Interpoliert").setText("Nein");
  }

  public void sendMqData() throws DataNotSubscribedException, SendSubscriptionNotConfirmed, OneSubscriptionPerSendData {
    AttributeGroup atg = davConnection.getDataModel().getAttributeGroup(ATG_MQ_VERKEHRSDATEN);
    Aspect asp = davConnection.getDataModel().getAspect(ASP_MQ_VERKEHRSDATEN);
    Collection<SystemObject> so = new ArrayList<>();
    so.add(davConnection.getDataModel().getObject("mq.MQ_1.090_HFB_NO"));
    so.add(davConnection.getDataModel().getObject("mq.MQ_1.090_HFB_SW"));

    DataDescription dd = new DataDescription(atg, asp);
    davConnection.subscribeReceiver(this,  so,  dd,  ReceiveOptions.delayed() , ReceiverRole.receiver());
     System.err.println("=== SUBSCRIBE RECEIVER ===");
     try { Thread.sleep(1000); } catch (InterruptedException dummy) { }
     System.err.println("=== SUBSCRIBE SENDER ===");
    davConnection.subscribeSender(this, so, dd, SenderRole.source());
    try { Thread.sleep(1000); } catch (InterruptedException dummy) { }
 
    Data d2 = davConnection.createData(atg);

    setStatusAndQS(d2.getItem("QKfz"));
    setStatusAndQS(d2.getItem("B"));
    setStatusAndQS(d2.getItem("BMax"));
    setStatusAndQS(d2.getItem("QB"));
    setStatusAndQS(d2.getItem("KB"));
    setStatusAndQS(d2.getItem("VDelta"));
    setStatusAndQS(d2.getItem("SKfz"));
    setStatusAndQS(d2.getItem("VgKfz"));
    setStatusAndQS(d2.getItem("KKfz"));
    setStatusAndQS(d2.getItem("KPkw"));
    setStatusAndQS(d2.getItem("KLkw"));
    setStatusAndQS(d2.getItem("QKfz"));
    setStatusAndQS(d2.getItem("QPkw"));
    setStatusAndQS(d2.getItem("QLkw"));
    setStatusAndQS(d2.getItem("VKfz"));
    setStatusAndQS(d2.getItem("VPkw"));
    setStatusAndQS(d2.getItem("VLkw"));
    setStatusAndQS(d2.getItem("ALkw"));

    Data d3 = davConnection.createData(atg);
    
    setStatusAndQS(d3.getItem("QKfz"));
    setStatusAndQS(d3.getItem("B"));
    setStatusAndQS(d3.getItem("BMax"));
    setStatusAndQS(d3.getItem("QB"));
    setStatusAndQS(d3.getItem("KB"));
    setStatusAndQS(d3.getItem("VDelta"));
    setStatusAndQS(d3.getItem("SKfz"));
    setStatusAndQS(d3.getItem("VgKfz"));
    setStatusAndQS(d3.getItem("KKfz"));
    setStatusAndQS(d3.getItem("KPkw"));
    setStatusAndQS(d3.getItem("KLkw"));
    setStatusAndQS(d3.getItem("QKfz"));
    setStatusAndQS(d3.getItem("QPkw"));
    setStatusAndQS(d3.getItem("QLkw"));
    setStatusAndQS(d3.getItem("VKfz"));
    setStatusAndQS(d3.getItem("VPkw"));
    setStatusAndQS(d3.getItem("VLkw"));
    setStatusAndQS(d3.getItem("ALkw"));
    
    d2.getItem("QKfz").getScaledValue("Wert").set(12);
    d2.getItem("VKfz").getScaledValue("Wert").set(15);
    d2.getItem("QPkw").getScaledValue("Wert").set(1);
    d2.getItem("VPkw").getScaledValue("Wert").set(12);
    d2.getItem("QLkw").getScaledValue("Wert").set(11);
    d2.getItem("VLkw").getScaledValue("Wert").set(17);
    d2.getItem("ALkw").getScaledValue("Wert").set(90);

    d3.getItem("QKfz").getScaledValue("Wert").set(12);
    d3.getItem("VKfz").getScaledValue("Wert").set(15);
    d3.getItem("QPkw").getScaledValue("Wert").set(1);
    d3.getItem("VPkw").getScaledValue("Wert").set(12);
    d3.getItem("QLkw").getScaledValue("Wert").set(11);
    d3.getItem("VLkw").getScaledValue("Wert").set(17);
    d3.getItem("ALkw").getScaledValue("Wert").set(90);

    sendData("mq.MQ_1.090_HFB_NO", ATG_MQ_VERKEHRSDATEN, ASP_MQ_VERKEHRSDATEN,d2);
    sendData("mq.MQ_1.090_HFB_SW", ATG_MQ_VERKEHRSDATEN, ASP_MQ_VERKEHRSDATEN,d3);

    System.err.println("Done (send MQ data).");

  }
  
  public void sendMqZustand() throws DataNotSubscribedException, SendSubscriptionNotConfirmed, OneSubscriptionPerSendData {
    AttributeGroup atg = davConnection.getDataModel().getAttributeGroup(ATG_MQ_ZUSTAND);
    Aspect asp = davConnection.getDataModel().getAspect(ASP_MQ_ZUSTAND);
    Collection<SystemObject> so = new ArrayList<>();
    so.add(davConnection.getDataModel().getObject("mq.MQ_1.090_HFB_NO"));
    so.add(davConnection.getDataModel().getObject("mq.MQ_1.090_HFB_SW"));

    DataDescription dd = new DataDescription(atg, asp);
    davConnection.subscribeReceiver(this,  so,  dd,  ReceiveOptions.delayed() , ReceiverRole.receiver());
    System.err.println("=== SUBSCRIBE RECEIVER ===");
    davConnection.subscribeSender(this, so, dd, SenderRole.source());
    System.err.println("=== SUBSCRIBE SENDER ===");
    try { Thread.sleep(1000); } catch (Exception dummy) { }
 
    Data d2 = davConnection.createData(atg);

    d2.getItem("Situation").asTextValue().setText("Stau");
    d2.getItem("T").asTimeValue().setMillis(60000);
    d2.getItem("Horizont").asTimeValue().setMillis(0);
    d2.getItem("Gte").getItem("Index").asScaledValue().set(0.9);
    d2.getItem("Gte").getItem("Verfahren").asTextValue().setText("Standard");

    Data d3 = davConnection.createData(atg);

    d3.getItem("Situation").asTextValue().setText("lebhafter Verkehr");
    d3.getItem("T").asTimeValue().setMillis(0);
    d3.getItem("Horizont").asTimeValue().setMillis(60000);
    d3.getItem("Gte").getItem("Index").asScaledValue().set(0.9);
    d3.getItem("Gte").getItem("Verfahren").asTextValue().setText("Standard");

    sendData("mq.MQ_1.090_HFB_NO", ATG_MQ_ZUSTAND, ASP_MQ_ZUSTAND,d2);
    sendData("mq.MQ_1.090_HFB_SW", ATG_MQ_ZUSTAND, ASP_MQ_ZUSTAND,d3);
    System.err.println("Done (send MQ status).");
  }

  public void sendFsData() throws DataNotSubscribedException, SendSubscriptionNotConfirmed, OneSubscriptionPerSendData, InterruptedException {
    AttributeGroup atg = davConnection.getDataModel().getAttributeGroup(ATG_FS_VERKEHRSDATEN);
    Aspect asp = davConnection.getDataModel().getAspect(ASP_FS_VERKEHRSDATEN);
    List<SystemObject> so = new ArrayList<>();
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_NO_1"));
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_NO_2"));
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_SW_1"));
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_SW_2"));

    DataDescription dd = new DataDescription(atg, asp);
    davConnection.subscribeReceiver(this,  so,  dd,  ReceiveOptions.delayed() , ReceiverRole.receiver());
    System.err.println("=== SUBSCRIBE RECEIVER ===");
    
    Thread.sleep(1000);
    
    davConnection.subscribeSender(this, so, dd, SenderRole.source());
    System.err.println("=== SUBSCRIBE SENDER ===");
    Thread.sleep(1000);
 
    Data d2 = davConnection.createData(atg);

    setStatusAndQS(d2.getItem("qKfz"));
    setStatusAndQS(d2.getItem("vKfz"));
    setStatusAndQS(d2.getItem("qLkw"));
    setStatusAndQS(d2.getItem("vLkw"));
    setStatusAndQS(d2.getItem("qPkw"));
    setStatusAndQS(d2.getItem("vPkw"));
    setStatusAndQS(d2.getItem("b"));
    setStatusAndQS(d2.getItem("tNetto"));
    setStatusAndQS(d2.getItem("sKfz"));
    setStatusAndQS(d2.getItem("vgKfz"));

    Data d3 = davConnection.createData(atg);
    
    setStatusAndQS(d3.getItem("qKfz"));
    setStatusAndQS(d3.getItem("vKfz"));
    setStatusAndQS(d3.getItem("qLkw"));
    setStatusAndQS(d3.getItem("vLkw"));
    setStatusAndQS(d3.getItem("qPkw"));
    setStatusAndQS(d3.getItem("vPkw"));
    setStatusAndQS(d3.getItem("b"));
    setStatusAndQS(d3.getItem("tNetto"));
    setStatusAndQS(d3.getItem("sKfz"));
    setStatusAndQS(d3.getItem("vgKfz"));

    d2.getItem("T").asTimeValue().setMillis(60000);
    d2.getItem("ArtMittelwertbildung").asTextValue().setText("unbekannt");
    d2.getItem("qKfz").getScaledValue("Wert").set(12);
    d2.getItem("vKfz").getScaledValue("Wert").set(15);
    d2.getItem("qPkw").getScaledValue("Wert").set(1);
    d2.getItem("vPkw").getScaledValue("Wert").set(12);
    d2.getItem("qLkw").getScaledValue("Wert").set(11);
    d2.getItem("vLkw").getScaledValue("Wert").set(17);


    d3.getItem("T").asTimeValue().setMillis(60000);
    d3.getItem("ArtMittelwertbildung").asTextValue().setText("unbekannt");
    d3.getItem("qKfz").getScaledValue("Wert").set(12);
    d3.getItem("vKfz").getScaledValue("Wert").set(15);
    d3.getItem("qPkw").getScaledValue("Wert").set(1);
    d3.getItem("vPkw").getScaledValue("Wert").set(12);
    d3.getItem("qLkw").getScaledValue("Wert").set(11);
    d3.getItem("vLkw").getScaledValue("Wert").set(17);

    sendData(so.get(0),atg,asp,d2);
    sendData(so.get(1),atg,asp,d3);
    sendData(so.get(2),atg,asp,d3);
    sendData(so.get(3),atg,asp,d2);
    
    System.err.println("Done (send FS data)");
  }


  public void sendFsZustand() throws DataNotSubscribedException, SendSubscriptionNotConfirmed, OneSubscriptionPerSendData {
    AttributeGroup atg = davConnection.getDataModel().getAttributeGroup(ATG_FS_ZUSTAND);
    Aspect asp = davConnection.getDataModel().getAspect(ASP_FS_ZUSTAND);
    List<SystemObject> so = new ArrayList<>();
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_NO_1"));
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_NO_2"));
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_SW_1"));
    so.add(davConnection.getDataModel().getObject("fs.MQ_1.090_HFB_SW_2"));

    DataDescription dd = new DataDescription(atg, asp);
    davConnection.subscribeReceiver(this,  so,  dd,  ReceiveOptions.delayed() , ReceiverRole.receiver());
    System.err.println("=== SUBSCRIBE RECEIVER ===");
    try { Thread.sleep(1000); } catch (InterruptedException dummy) { }
    davConnection.subscribeSender(this, so, dd, SenderRole.source());
    System.err.println("=== SUBSCRIBE SENDER ===");
    try { Thread.sleep(1000); } catch (InterruptedException dummy) { }
 
    Data d2 = davConnection.createData(atg);

    setStatusAndQS(d2);
    d2.getItem("T").asTimeValue().setMillis(0);
    d2.getItem("Situation").asTextValue().setText("lebhafter Verkehr");
    d2.getItem("Horizont").asTimeValue().setMillis(0);

    sendData(so.get(0),atg,asp,d2);
    sendData(so.get(1),atg,asp,d2);
    sendData(so.get(2),atg,asp,d2);
    sendData(so.get(3),atg,asp,d2);
    
    System.err.println("Done (send FS status).");
  }
  
  protected void sendUfds(String objStr, String atgStr, String field, Object value) throws OneSubscriptionPerSendData, DataNotSubscribedException, SendSubscriptionNotConfirmed {
    System.err.println(" => " + atgStr);
    AttributeGroup atg = davConnection.getDataModel().getAttributeGroup(atgStr);
    Aspect asp = davConnection.getDataModel().getAspect(ASP_UFDS);
    List<SystemObject> so = new ArrayList<>(); 
    so.add(davConnection.getDataModel().getObject(objStr));

    DataDescription dd = new DataDescription(atg, asp);
    davConnection.subscribeReceiver(this,  so,  dd,  ReceiveOptions.delayed() , ReceiverRole.receiver());
    System.err.println("=== SUBSCRIBE RECEIVER ===");
    try { Thread.sleep(1000); } catch (InterruptedException dummy) { }
    davConnection.subscribeSender(this, so, dd, SenderRole.source());
    System.err.println("=== SUBSCRIBE SENDER ===");
    try { Thread.sleep(1000); } catch (InterruptedException dummy) { }

    Data d2 = davConnection.createData(atg);
    d2.getItem("T").asTimeValue().setMillis(60000);
    setStatusAndQS(d2.getItem(field));
    if (value instanceof Integer) {
      d2.getItem(field).getScaledValue("Wert").set((Integer) value);
    } else {
      d2.getItem(field).getTextValue("Wert").setText(value.toString());        
    }
    sendData(so.get(0),atg,asp,d2);
  }
  
  public void sendUmfelddata() throws DataNotSubscribedException, SendSubscriptionNotConfirmed, OneSubscriptionPerSendData {
    sendUfds("ufds.x1.007","atg.ufdsWindRichtung","WindRichtung",122);
    sendUfds("ufds.x1.006","atg.ufdsWindGeschwindigkeitSpitzenWert","WindGeschwindigkeitSpitzenWert", 0);
    sendUfds("ufds.x1.005","atg.ufdsWindGeschwindigkeitMittelWert","WindGeschwindigkeitMittelWert", 12);
    sendUfds("ufds.x1.120","atg.ufdsWasserFilmDicke","WasserFilmDicke", 12);
    sendUfds("ufds.x1.012","atg.ufdsTaupunktTemperatur","TaupunktTemperatur", 1);
    sendUfds("ufds.x1.035","atg.ufdsSichtWeite","SichtWeite",13);
    sendUfds("ufds.x1.009","atg.ufdsSchneeHhe","SchneeHhe",13);
    sendUfds("ufds.x1.125","atg.ufdsRestSalz","RestSalz",13);
    sendUfds("ufds.x1.004","atg.ufdsRelativeLuftFeuchte","RelativeLuftFeuchte",56);
    sendUfds("ufds.x1.078","atg.ufdsNiederschlagsIntensitt","NiederschlagsIntensitt",22);
    sendUfds("ufds.x1.068","atg.ufdsNiederschlagsArt","NiederschlagsArt","Hagel WMO1");
    sendUfds("ufds.x1.088","atg.ufdsNiederschlagsMenge","NiederschlagsMenge",24);
    sendUfds("ufds.x1.001","atg.ufdsLuftTemperatur","LuftTemperatur",12);    
    sendUfds("ufds.x1.003","atg.ufdsGefrierTemperatur","GefrierTemperatur",1);    
    sendUfds("ufds.x1.070","atg.ufdsFahrBahnOberFlchenZustand","FahrBahnOberFlchenZustand","feucht");    
    sendUfds("ufds.x1.002","atg.ufdsFahrBahnOberFlchenTemperatur","FahrBahnOberFlchenTemperatur",34);    
  }
  
  public void teardown() {
    davConnection.disconnect(false, "Hahaha");
  }

  public static void main(String[] args) throws MissingParameterException, CommunicationError, ConnectionException, InconsistentLoginException, DataNotSubscribedException, SendSubscriptionNotConfirmed, OneSubscriptionPerSendData {
    InjiziereTestdaten001 instance = new InjiziereTestdaten001();
    instance.initialize(args);
    
    try {
      instance.sendMqData();
      instance.sendMqZustand();
      instance.sendFsData();
      instance.sendFsZustand();
      instance.sendUmfelddata();
    } catch (Exception ex) {
      ex.printStackTrace();
    }
    // instance.teardown();
  }

  @Override
  public void dataRequest(SystemObject arg0, DataDescription arg1, byte arg2) {
    System.err.println("DataReq." + arg0.getPidOrNameOrId() + " " + arg1.getAttributeGroup().getPidOrNameOrId() + " " + arg1.getAspect().getPidOrNameOrId() + " :" + arg2);
  }

  @Override
  public boolean isRequestSupported(SystemObject arg0, DataDescription arg1) {
    return false;
  }

  @Override
  public void update(ResultData[] arg0) {
    // TODO Auto-generated method stub
    
  }
}
