package de.bsvrz.datex2.test.cases.old;
import java.io.ByteArrayOutputStream;
import java.io.StringReader;
import java.lang.reflect.Field;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;

import javax.naming.ConfigurationException;
import javax.xml.XMLConstants;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.Marshaller;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.namespace.QName;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.bsvrz.datex2.test.mockclasses.MockAttributeGroup;
import de.bsvrz.datex2.test.mockclasses.MockData;
import de.bsvrz.datex2.test.mockclasses.MockDataModel;
import de.bsvrz.datex2.test.mockclasses.MockDataNumberValue;
import de.bsvrz.datex2.test.mockclasses.MockDataTextValue;
import de.bsvrz.datex2.test.mockclasses.MockDavConnection;
import de.bsvrz.datex2.test.mockclasses.MockKonfigUndPersistenzModul;
import de.bsvrz.datex2.test.mockclasses.MockObjectSet;
import de.bsvrz.datex2.test.mockclasses.MockObjekteAtgAsp;
import de.bsvrz.datex2.test.mockclasses.MockPluginExport;
import de.bsvrz.datex2.test.mockclasses.MockSystemObject;
import de.bsvrz.datex2.test.mockclasses.MockSystemObjectType;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.datex2.allgemein.ObjekteAtgAsp;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.FSPredefinedLocation;
import de.bsvrz.kex.datex2.plugins.konvertierung.impl.WerkzeugSammlung;

public class WBKonvertierung003s {
  private static final String TYP_FAHRSTREIFEN = "typ.fahrStreifen";
  private static final String ATG_VERKEHRSDATEN = "atg.verkehrsDatenKurzZeit";
  private static final String SCHEMA_URI = "file:///d:/home/workspace/KExDatexII/org/DATEXII_DaV-MDM-003_stat.xsd";
  
  /** Datex II Namensraum */
  private final String DATEX2_NAMESPACE = "http://datex2.eu/schema/2/2_0";

  /** Root-Tag des Datex II Formats */
  private final String DATEX2_ROOTTAG = "d2LogicalModel";

  MockSystemObject ordinaryParentMQ;
  MockSystemObject faultParentMQ1;
  MockSystemObject faultParentMQ2;
  
  MockDataModel ordinaryDataModel;
  
  MockObjekteAtgAsp ordinaryObjSet = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet1 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet2 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet3 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp faultObjSet4 = new MockObjekteAtgAsp();
  MockObjekteAtgAsp emptyObjSet = new MockObjekteAtgAsp();
  private Schema schema;
  
  enum CanBeNull { CAN_BE_NULL, CANNOT_BE_NULL };

  @Before
  public void createDataSets() throws Exception {
    createDataModel();
    createParentMQ();

    AttributeGroup atgFS = ordinaryDataModel.getAttributeGroup("atg.fahrStreifen");

    ordinaryObjSet = new MockObjekteAtgAsp();
    ordinaryObjSet.createMockTyp(TYP_FAHRSTREIFEN, "Fahrstreifen");
    ordinaryObjSet.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");

    MockSystemObject so = new MockSystemObject("pid.fs01", "");
    so.setDataModel(ordinaryDataModel);

    MockData dx = new MockData();
    MockDataTextValue dxt = new MockDataTextValue();
    dxt.setText("HFS");
    dx.setTextValue("Lage", dxt);
    so.getConfigurationDataMap().put(atgFS.getPid(), dx);

    MockSystemObject so2 = new MockSystemObject("pid.fs02", "");
    so2.setDataModel(ordinaryDataModel);
 
    MockData dx2 = new MockData();
    MockDataTextValue dxt2 = new MockDataTextValue();
    dxt2.setText("HFS");
    dx2.setTextValue("Lage", dxt);
    so2.getConfigurationDataMap().put(atgFS.getPid(), dx2);

    faultObjSet1.getSystemObjekte().add(so);

    ordinaryObjSet.getSystemObjekte().add(so);
    ordinaryObjSet.getSystemObjekte().add(so2);
    
    faultObjSet1 = new MockObjekteAtgAsp();
    faultObjSet1.createMockTyp("typ.nichtExistent", "Messquerschnitt");
    faultObjSet1.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");

    faultObjSet2 = new MockObjekteAtgAsp();
    faultObjSet2.createMockTyp(TYP_FAHRSTREIFEN, "Messquerschnitt");
    faultObjSet2.createAttributeGroup("atg.nichtExistent", "Verkehrsdaten");
    
    faultObjSet3 = new MockObjekteAtgAsp();
    faultObjSet3.createMockTyp(TYP_FAHRSTREIFEN, "Messquerschnitt");
    faultObjSet3.createAttributeGroup("atg.nichtExistent", "Verkehrsdaten");
    MockSystemObject so3 = new MockSystemObject("pid.mq03", "");
    so3.setDataModel(ordinaryDataModel);
    faultObjSet3.getSystemObjekte().add(so3);
    System.err.println("so3: " + so3.getConfigurationDataMap());

    faultObjSet4 = new MockObjekteAtgAsp();
    faultObjSet4.createMockTyp(TYP_FAHRSTREIFEN, "Messquerschnitt");
    faultObjSet4.createAttributeGroup(ATG_VERKEHRSDATEN, "Verkehrsdaten");
    MockSystemObject so4 = new MockSystemObject("pid.mq04", "");
    so4.setDataModel(ordinaryDataModel);
    MockData dp4 = new MockData();
    MockDataTextValue nv4 = new MockDataTextValue();
    nv4.setText("HAHA");
    dp4.setTextValue("x", nv4);
    dp4.setTextValue("y", nv4);
//    so4.getConfigurationDataMap().put(atgPunkt, dp4);

    faultObjSet4.getSystemObjekte().add(so4);
    faultObjSet4.getSystemObjekte().add(so2);
        
    emptyObjSet = new MockObjekteAtgAsp();

    URL schemaFile = new URL(SCHEMA_URI);
    SchemaFactory schemaFactory = SchemaFactory
        .newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
    schema = schemaFactory.newSchema(schemaFile);
  }

  public void createDataModel() {
    ordinaryDataModel = new MockDataModel();
    MockAttributeGroup atgPunkt = new MockAttributeGroup("atg.punktKoordinaten");
    ordinaryDataModel.getAttributeGroupMap().put("atg.punktKoordinaten", atgPunkt);
  }
  
  public void createParentMQ() {
    Collection<SystemObject> solist = new ArrayList<>();
    MockSystemObject mso = new MockSystemObject("mq01", "");
    solist.add(mso);
    MockAttributeGroup atgPunkt = new MockAttributeGroup("atg.punktKoordinaten");
    ordinaryDataModel.getAttributeGroupMap().put("atg.punktKoordinaten", atgPunkt);
    mso.setDataModel(ordinaryDataModel);
    MockData dp = new MockData();
    MockDataNumberValue nv = new MockDataNumberValue();
    nv.set(34);
    dp.setNumberValue("x", nv);
    dp.setNumberValue("y", nv);
    mso.getConfigurationDataMap().put(atgPunkt.getPid(), dp);

    MockAttributeGroup atgFS = new MockAttributeGroup("atg.fahrStreifen");
    ordinaryDataModel.getAttributeGroupMap().put("atg.fahrStreifen", atgFS);

    MockData dx = new MockData();
    MockDataTextValue dxt = new MockDataTextValue();
    dxt.setText("HFS");
    dx.setTextValue("Lage", dxt);
    mso.getConfigurationDataMap().put(atgFS.getPid(), dx);
    
    ordinaryDataModel.addObjects(new MockSystemObjectType("typ.messQuerschnitt", ""), solist);

    ordinaryParentMQ = mso;
    
  }
  
  public Object apply(MockPluginExport plugin,ObjekteAtgAsp objekteAtgAsp) throws MissingParameterException, ConfigurationChangeException {
    Collection<ObjekteAtgAsp> objList = new ArrayList<>();
    objList.add(objekteAtgAsp);
    MockObjectSet mos = (MockObjectSet) ordinaryParentMQ.newObjectSet("FahrStreifen");
    for (ObjekteAtgAsp a : objList) {
      for (SystemObject x : a.getSystemObjekte()) {
        mos.add(x);
      }
    }

//    ordinaryDataModel.addObjects(new MockSystemObjectType("typ.messQuerschnitt", ""), ordinaryParentMQ);

//    MockDataModel dm = (MockDataModel) objekteAtgAsp.getSystemObjekte().get(0).getDataModel();
//    MockDavConnection davConnection = new MockDavConnection(null, dm);
//    plugin.setDaVConnection(davConnection);
//    
//    Collection<SystemObject> solist = new ArrayList<>();
//    MockSystemObject mso = new MockSystemObject("mq01");
//    solist.add(mso);
//    MockAttributeGroup atgPunkt = new MockAttributeGroup("atg.punktKoordinaten");
//    dm.getAttributeGroupMap().put("atg.punktKoordinaten", atgPunkt);
//    mso.setDataModel(dm);
//    MockData dp = new MockData();
//    MockDataNumberValue nv = new MockDataNumberValue();
//    nv.set(34);
//    dp.setNumberValue("x", nv);
//    dp.setNumberValue("y", nv);
//    mso.getConfigurationDataMap().put(atgPunkt, dp);
//    MockObjectSet mos = (MockObjectSet) mso.newObjectSet("FahrStreifen");
//    mos.add(objekteAtgAsp.getSystemObjekte().get(0));
//   
//    dm.addObjects(new MockSystemObjectType("typ.messQuerschnitt", ""), solist);
    
    
    
    try {
      plugin.getDatexKonvertierungStatisch().initialisiere(objList);
    } catch (ConfigurationException e) {
      e.printStackTrace();
      return null;
    }
     
    
    
    for (ObjekteAtgAsp a : objList) {
      System.err.println(a.getSystemObjekte().size());
    }
    
    return plugin.getDatexKonvertierungStatisch().getDatexII(objList);
  }
  
  public MockPluginExport setupPluginFull() throws MissingParameterException {
    MockPluginExport mockPluginExport = new MockPluginExport();
    MockKonfigUndPersistenzModul konfig = new MockKonfigUndPersistenzModul();
    MockKonfigUndPersistenzModul persistenz = new MockKonfigUndPersistenzModul();
    mockPluginExport.setKonfigurationsModul(konfig);
    mockPluginExport.setPersistenzModul(persistenz);    
    
    mockPluginExport.setDatexKonvertierungStatisch(new FSPredefinedLocation(mockPluginExport));

    MockDavConnection davConnection = new MockDavConnection(null, ordinaryDataModel);
    mockPluginExport.setDaVConnection(davConnection);
    
        
    konfig.putData("National-Identifier", "TestNI");
    konfig.putData("LocationGroupId", "TestGroupId");
    konfig.putData("StandardIntervall", "3.0");
    
    return mockPluginExport;
  }
  
  @Test
  public void test001() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CANNOT_BE_NULL));
  }

  @Test
  public void test002() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet1);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test003() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet2);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test004() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet3);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }

  @Test
  public void test005() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, faultObjSet4);
    System.err.println(xml);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
  }
  
  @Test
  public void test011() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getPersistenzModul()).putData("PredefinedLocationVersion", "MOCKVERSION");
    Object xml = apply(plugin, faultObjSet1);
    Assert.assertTrue(validate(plugin, xml,CanBeNull.CAN_BE_NULL));
    Assert.assertEquals("MOCKVERSION", ((FSPredefinedLocation) plugin.getDatexKonvertierungStatisch()).getPredefinedLocationVersion()); 
  }
  
  @Test
  public void test012() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("National-Identifier");
    Object xml = apply(plugin, ordinaryObjSet);
    Assert.assertNull(xml);
  }
  
  @Test
  public void test013() throws Exception {
    MockPluginExport plugin = setupPluginFull();
    ((MockKonfigUndPersistenzModul) plugin.getKonfigurationsModul()).removeData("LocationGroupId");
    Object xml = apply(plugin, ordinaryObjSet);
    Assert.assertNull(xml);
  }

  @Test
  public void test017() throws Exception {
    for (Field f : WerkzeugSammlung.class.getDeclaredFields()) {
      if (f.getType().equals(DatatypeFactory.class)) {
        f.setAccessible(true);
        f.set(null,null);
        System.err.println("Reflection: Found and patched field.");
      }
    }
 
    System.getProperties().put("javax.xml.datatype.DatatypeFactory", "Object");
    MockPluginExport plugin = setupPluginFull();
    Object xml = apply(plugin, ordinaryObjSet);
    System.getProperties().remove("javax.xml.datatype.DatatypeFactory");
    Assert.assertNull(xml);
  }

  public boolean validate(MockPluginExport plugin, Object xml, CanBeNull canBeNull) throws Exception {
    if (xml == null) {
      return canBeNull.equals(CanBeNull.CAN_BE_NULL); 
    }
    Marshaller marshaller = plugin.getDatexKonvertierungStatisch().getJAXBContext().createMarshaller();
    marshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, new Boolean(true));

    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml),System.out);

    ByteArrayOutputStream ba = new ByteArrayOutputStream();
    marshaller.marshal(new JAXBElement<Object>(new QName(DATEX2_NAMESPACE,DATEX2_ROOTTAG), Object.class, xml), ba);
    Source xmls = new StreamSource(new StringReader(ba.toString()));
    Validator validator = schema.newValidator();
    validator.validate(xmls);

    return true;
  }
}
